package main

import (
	"io"
	"log"
	"net/http"
	"os"
	"sevenkeys/database"
	"sevenkeys/database/entities"
	"sevenkeys/database/operations"
	"sevenkeys/scryfall/methods"
	"sevenkeys/scryfall/types"
	"time"
)

func check(err error) {
	if err != nil {
		log.Fatal(err)
	}
}

func main() {
	log.Printf("Getting bulk data listing...")
	allCardsBulkData, err := methods.GetBulkDataByType(types.BulkDataTypeAllCards)
	check(err)
	log.Printf("Got bulk data listing.")

	log.Printf("Checking if we need to download bulk data...")
	updatedAtTimestamp, err := time.Parse(types.ScryfallTimestampFormat, allCardsBulkData.UpdatedAt)
	check(err)
	// Remove the fractional seconds component from the downloaded timestamp because it isn't saved in the cache table
	updatedAtTimestamp = updatedAtTimestamp.Truncate(time.Second)

	db := database.GetDatabaseFromConfig("config.json")

	cachedFileTimestamp, err := operations.GetCacheTimestampByType(db, entities.CacheTypeAllCardsBulkData)
	check(err)

	if updatedAtTimestamp.After(cachedFileTimestamp) {
		log.Printf("Cached time: ", cachedFileTimestamp)
		log.Printf("Update time: ", updatedAtTimestamp)
		log.Printf("Bulk data has been updated since last cache, redownloading.")
		cacheDir := "cache"

		err = os.RemoveAll(cacheDir)
		check(err)

		err := os.Mkdir(cacheDir, os.ModePerm)
		check(err)

		log.Printf("Downloading bulk card data...")
		bulkCardsResponse, err := http.Get(allCardsBulkData.DownloadUri)
		check(err)
		log.Printf("Downloaded bulk card data.")

		log.Printf("Writing card data to cache file...")
		cacheFilename := cacheDir + "/all-cards.json"
		cacheFile, err := os.Create(cacheFilename)
		check(err)

		defer bulkCardsResponse.Body.Close()
		defer cacheFile.Close()
		io.Copy(cacheFile, bulkCardsResponse.Body)
		log.Printf("Cache file written.")

		log.Printf("Recording timestamp...")
		err = operations.InsertOrUpdateCacheTimestampByType(db, entities.CacheTypeAllCardsBulkData, updatedAtTimestamp)
		check(err)
		log.Printf("Timestamp recorded.")
	} else {
		log.Printf("Bulk data has not been updated. Skipping download.")
	}

	// Marshal the bulk cards json from the cache file into a struct
	// Import the json into the database

	/* OLD CODE
	db := database.GetDatabaseFromConfig("config.json")
	allCards := methods.GetAllCards()
	operations.InsertCards(db, allCards)
	*/
}
